@@ -1,5 +1,5 @@
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-   Copyright (c) 2013-2020 The plumed team
+   Copyright (c) 2013-2021 The plumed team
    (see the PEOPLE file at the root of the distribution for a list of names)
 
    See http://www.plumed.org for more information.
@@ -26,7 +26,7 @@
 #include "core/Value.h"
 #include "tools/OFile.h"
 #include "tools/PDB.h"
-#include "core/SetupMolInfo.h"
+#include "core/GenericMolInfo.h"
 
 namespace PLMD {
 
