@@ -564,7 +564,7 @@ int MD_LinearExpansionPES::main( FILE* in, FILE* out, PLMD::Communicator& pc) {
   for(unsigned int k=0; k<dim; k++) {
     positions[0][k] = initPos[inter.Get_rank()][k];
     if(periodic[k]) {
-      positions[0][k] = positions[0][k] - floor(positions[0][k]/interval_range[k]+0.5)*interval_range[k];
+      positions[0][k] = positions[0][k] - floor((positions[0][k]-interval_min[k])/interval_range[k])*interval_range[k];
     }
     else {
       if(positions[0][k]>interval_max[k]) {positions[0][k]=interval_max[k];}
@@ -590,9 +590,9 @@ int MD_LinearExpansionPES::main( FILE* in, FILE* out, PLMD::Communicator& pc) {
     int step_tmp = 0;
     plumed->cmd("setStep",&step_tmp);
     plumed->cmd("setMasses",&masses[0]);
-    plumed->cmd("setForces",&forces[0]);
+    plumed->cmd("setForces",&forces[0][0]);
     plumed->cmd("setEnergy",&potential);
-    plumed->cmd("setPositions",&positions[0]);
+    plumed->cmd("setPositions",&positions[0][0]);
     plumed->cmd("calc");
   }
 
@@ -614,7 +614,7 @@ int MD_LinearExpansionPES::main( FILE* in, FILE* out, PLMD::Communicator& pc) {
       positions[0][k] = positions[0][k] + tstep*velocities[0][k];
 
       if(periodic[k]) {
-        positions[0][k] = positions[0][k] - floor(positions[0][k]/interval_range[k]+0.5)*interval_range[k];
+        positions[0][k] = positions[0][k] - floor((positions[0][k]-interval_min[k])/interval_range[k])*interval_range[k];
       }
       else {
         if(positions[0][k]>interval_max[k]) {
@@ -635,9 +635,9 @@ int MD_LinearExpansionPES::main( FILE* in, FILE* out, PLMD::Communicator& pc) {
       plumedWantsToStop=0;
       plumed->cmd("setStep",&istepplusone);
       plumed->cmd("setMasses",&masses[0]);
-      plumed->cmd("setForces",&forces[0]);
+      plumed->cmd("setForces",&forces[0][0]);
       plumed->cmd("setEnergy",&potential);
-      plumed->cmd("setPositions",&positions[0]);
+      plumed->cmd("setPositions",&positions[0][0]);
       plumed->cmd("setStopFlag",&plumedWantsToStop);
       plumed->cmd("calc");
       //if(istep%2000==0) plumed->cmd("writeCheckPointFile");
